'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDialect = getDialect;
exports.isPostgres = isPostgres;
exports.isMySql = isMySql;
exports.isSqlite = isSqlite;
exports.isKnexQueryBuilder = isKnexQueryBuilder;
exports.overwriteForDatabase = overwriteForDatabase;

var _defineProperty = require('babel-runtime/core-js/object/define-property');

var _defineProperty2 = _interopRequireDefault(_defineProperty);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var OVERWRITE_FOR_DATABASE_KEY = '@overwriteForDatabase';

function getDialect(knex) {
  return knex.client.dialect;
}

function isPostgres(knex) {
  return getDialect(knex) === 'postgresql';
}

function isMySql(knex) {
  return getDialect(knex) === 'mysql';
}

function isSqlite(knex) {
  return getDialect(knex) === 'sqlite3';
}

function isKnexQueryBuilder(knexQueryBuilder) {
  return knexQueryBuilder && knexQueryBuilder.client && _lodash2.default.isString(knexQueryBuilder.client.dialect);
}

function overwriteForDatabase(input) {
  // If there is no input or if the input is a function, we assume that the
  // decorator was applied to a class instead of a method.
  var isClassDecorator = _lodash2.default.isUndefined(input) || _lodash2.default.isFunction(input);

  if (isClassDecorator) {
    // In case of class decorator, the input should be a function that returns
    // a knex instance that the method version can use.
    var getKnex = input;

    if (_lodash2.default.isUndefined(getKnex)) {
      // The default getter attempts to call a function called `knex`.
      getKnex = function getKnex(inst) {
        return inst.knex();
      };
    }

    return overwriteForDatabaseClass(getKnex);
  } else {
    return overwriteForDatabaseMethod(input);
  }
}

function overwriteForDatabaseClass(getKnex) {
  return function (constructor) {
    if (constructor[OVERWRITE_FOR_DATABASE_KEY]) {
      // Knex getter is already registered. Do nothing.
      return;
    }

    (0, _defineProperty2.default)(constructor, OVERWRITE_FOR_DATABASE_KEY, {
      enumerable: false,
      writable: false,
      value: { getKnex: getKnex }
    });
  };
}

function overwriteForDatabaseMethod(input) {
  return function (target, property, descriptor) {
    var methodNameByDialect = input;
    var defaultImpl = descriptor.value;

    descriptor.value = function () {
      var knex = this.constructor[OVERWRITE_FOR_DATABASE_KEY].getKnex(this);
      var dialect = getDialect(knex);

      // Call the correct method based on the dialect.
      if (dialect in methodNameByDialect) {
        var methodName = methodNameByDialect[dialect];
        return this[methodName].apply(this, arguments);
      } else {
        return defaultImpl.apply(this, arguments);
      }
    };
  };
}